"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _app = require("meetup-web-mocks/lib/app");

var api = _interopRequireWildcard(require("../sync/apiActionCreators"));

var _util = require("./util");

var _cacheActionCreators = require("./cacheActionCreators");

var _ = _interopRequireDefault(require("./"));

var MOCK_QUERY = (0, _app.mockQuery)(_app.MOCK_RENDERPROPS);
var MOCK_SUCCESS_ACTION = api.success({
  query: api.get(MOCK_QUERY).payload[0],
  response: _app.MOCK_API_RESULT[0]
});
var apiRequestAction = api.get(MOCK_QUERY);
var fakeStore = {
  getState: function getState() {
    return {
      api: {},
      config: {
        member: {
          id: 0
        }
      }
    };
  },
  dispatch: function dispatch() {},
  subscribe: function subscribe() {}
};

function makeCacheEpic() {
  return Promise.resolve((0, _.default)((0, _util.makeCache)()));
}

function populateCacheEpic(CacheEpic) {
  // set the cache with API_RESP_SUCCESS
  return CacheEpic(MOCK_SUCCESS_ACTION, fakeStore).then(function () {
    return CacheEpic;
  });
}

function clearCacheEpic(CacheEpic) {
  // clear the cache with CACHE_CLEAR
  return CacheEpic({
    type: _cacheActionCreators.CACHE_CLEAR
  }, fakeStore).then(function () {
    return CacheEpic;
  });
}

var testForEmptyCache = function testForEmptyCache() {
  var action = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : apiRequestAction;
  return function (CacheEpic) {
    return CacheEpic(action, fakeStore).then(function (actions) {
      return expect(actions).toHaveLength(0);
    });
  };
};

var testForPopulatedCache = function testForPopulatedCache() {
  var action = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : apiRequestAction;
  return function (CacheEpic) {
    return CacheEpic(action, fakeStore).then(function (actions) {
      return expect(actions.map(function (_ref) {
        var type = _ref.type;
        return type;
      })).toContain(_cacheActionCreators.CACHE_SUCCESS);
    });
  };
};

describe('getCacheEpic', function () {
  it('does not pass through arbitrary actions', function () {
    return (0, _.default)()({
      type: 'asdf'
    }).then(function (actions) {
      return expect(actions).toHaveLength(0);
    });
  });
  it('does not emit CACHE_SUCCESS when no cache hit from API_REQ', function () {
    return makeCacheEpic().then(testForEmptyCache());
  });
  it('emits CACHE_SUCCESS when there is a cache hit for API_REQ', function () {
    return makeCacheEpic().then(populateCacheEpic) // also indirectly testing for successful cache set on API_RESP_SUCCESS
    .then(testForPopulatedCache());
  });
  it('does not emit CACHE_SUCCESS after CACHE_CLEAR is dispatched', function () {
    return makeCacheEpic().then(populateCacheEpic).then(clearCacheEpic).then(testForEmptyCache());
  });
});
//# sourceMappingURL=index.test.js.map