"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deleteEpic = exports.postEpic = void 0;

var api = _interopRequireWildcard(require("../sync/apiActionCreators"));

/**
 * Mutate epic  provides a generic interface for triggering POST and DELETE requests
 * and dispatching particular actions with the API response. The POST or DELETE action
 * must follow this structure:
 *
 * ```
 * {
 *   type: 'POST_<SOMETHING>' (or '<SOMETHING>_POST'),
 *   payload: {
 *     query: a 'reactive query' object that maps to an API call,
 *     onSuccess: a callback that takes the API response as an argument, and
 *       returns an action object. The middleware takes care of dispatch
 *     onError: a callback that takes an Error argument and returns an action
 *       object
 *   }
 * }
 * ```
 *
 * This structure usually allows the success/error handling code to be bundled
 * alongside the POST/DELETE action creator, with the expectation that all response
 * processing can be done there
 *
 * @deprecated
 * @module MutateEpic
 */

/*
 * Delegate method-specific request actions (e.g. 'POST_MY_FORM') to the sync
 * middleware by modifying the request actions' query payloads to include
 * request method properties
 * @deprecated
 */
var _getMethodEpic = function _getMethodEpic(method) {
  var matchAction = function matchAction(_ref) {
    var type = _ref.type;
    return type.endsWith("_".concat(method.toUpperCase())) || type.startsWith("".concat(method.toUpperCase(), "_"));
  };

  return function (action) {
    if (!matchAction(action)) {
      return Promise.resolve([]);
    }

    var type = action.type,
        _action$payload = action.payload,
        query = _action$payload.query,
        onSuccess = _action$payload.onSuccess,
        onError = _action$payload.onError; // Webpack will make sure process.env.NODE_ENV is populated on client

    if (process.env.NODE_ENV !== 'production') {
      // using a dev endpoint, render a deprecation warning
      console.warn("This application is using Post/Delete middleware through ".concat(type, ".\n\tSee the platform Queries Recipes docs for refactoring options:\n\thttps://github.com/meetup/meetup-web-platform/blob/main/docs/Queries.md#recipes"));
    }

    var actionCreator = method === 'delete' ? 'del' : method;
    var meta = {
      onSuccess: onSuccess,
      onError: onError
    };
    return Promise.resolve([api[actionCreator](query, meta)]);
  };
};

var postEpic = _getMethodEpic('post');

exports.postEpic = postEpic;

var deleteEpic = _getMethodEpic('delete');

exports.deleteEpic = deleteEpic;
//# sourceMappingURL=index.js.map