"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.api = api;
exports.responseToState = exports.getListState = exports.filterKeys = exports.DEFAULT_API_STATE = void 0;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _objectSpread4 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _mwpRouter = require("mwp-router");

var _apiActionCreators = require("./sync/apiActionCreators");

var _cacheActionCreators = require("./cache/cacheActionCreators");

var DEFAULT_API_STATE = {
  inFlight: []
};
exports.DEFAULT_API_STATE = DEFAULT_API_STATE;

var filterKeys = function filterKeys(obj, whitelist, keys) {
  return Object.keys(obj).reduce(function (newObj, key) {
    if (!keys.includes(key) || whitelist.includes(key)) {
      newObj[key] = obj[key];
    }

    return newObj;
  }, {});
};

exports.filterKeys = filterKeys;

/*
 * Queries can specify 'list' response handling - response values will be
 * stored in a `dyanmicRef` in state. If a `list.merge` param is specified,
 * the response will be merged with the existing list state stored in `dynamicRef`
 */
var getListState = function getListState(state, _ref) {
  var response = _ref.response,
      query = _ref.query;

  if (!response || !query.list) {
    // no list no problem
    return {};
  }

  var _query$list = query.list,
      dynamicRef = _query$list.dynamicRef,
      merge = _query$list.merge; // this query should be treated as a list-building query
  // list can be either a root query result (response.value) or be under `value` field of the result (response.value.value)

  var newList = [];

  if (response.value instanceof Array) {
    newList = response.value;
  } else if (response.value && response.value.value instanceof Array) {
    newList = response.value.value;
  }

  if (!merge) {
    // no merge rules, so just make a new list
    return (0, _defineProperty2.default)({}, dynamicRef, {
      value: newList,
      query: query
    });
  } // do some smart merging


  var idTest = merge.idTest,
      sort = merge.sort,
      isReverse = merge.isReverse; // remove anything in old list that is part of new list

  var oldList = ((state[dynamicRef] || {}).value || []).filter(function (valOld) {
    return !newList.find(function (valNew) {
      return idTest(valOld, valNew);
    });
  }); // combine the new list and the old list

  var mergedList = isReverse ? [].concat((0, _toConsumableArray2.default)(newList), (0, _toConsumableArray2.default)(oldList)) : [].concat((0, _toConsumableArray2.default)(oldList), (0, _toConsumableArray2.default)(newList)); // sort is optional

  if (sort) {
    mergedList.sort(sort);
  }

  return (0, _defineProperty2.default)({}, dynamicRef, {
    value: mergedList,
    query: query
  });
};

exports.getListState = getListState;

var responseToState = function responseToState(state, _ref4) {
  var response = _ref4.response,
      query = _ref4.query;
  return (0, _objectSpread4.default)((0, _defineProperty2.default)({}, query.ref, (0, _objectSpread4.default)({}, response, {
    query: query
  })), getListState(state, {
    response: response,
    query: query
  }));
};
/*
 * The primary reducer for data provided by the API
 */


exports.responseToState = responseToState;

function api() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_API_STATE;
  var action = arguments.length > 1 ? arguments[1] : undefined;

  switch (action.type) {
    case _mwpRouter.LOCATION_CHANGE:
      {
        var inFlight = state.inFlight,
            fail = state.fail,
            refs = (0, _objectWithoutProperties2.default)(state, ["inFlight", "fail"]);
        return Object.keys(refs).reduce(function (cleanState, ref) {
          // throw out data from queries that are not 'GET' - it should not be kept in state
          var queryMethod = ((state[ref].query || {}).meta || {}).method;

          if (queryMethod && queryMethod !== 'get') {
            return cleanState;
          }

          return (0, _objectSpread4.default)({}, cleanState, (0, _defineProperty2.default)({}, ref, state[ref]));
        }, {
          inFlight: inFlight,
          fail: fail
        });
      }

    case _apiActionCreators.API_REQ:
      {
        var requestRefs = (action.payload || []).map(function (_ref5) {
          var ref = _ref5.ref;
          return ref;
        });

        var _inFlight = state.inFlight.filter(function (ref) {
          return !requestRefs.includes[ref];
        }) // clean out current duplicates
        .concat(requestRefs); // add new requested refs


        if ((action.meta || {}).logout) {
          // clear app state during logout
          return (0, _objectSpread4.default)({}, DEFAULT_API_STATE, {
            inFlight: _inFlight
          });
        } // remove any `ref`s that are being refreshed - eliminate stale data
        // however, keep any refs whose values are not expected to change


        var refWhitelist = (action.meta || {}).retainRefs || [];
        var newState = (0, _objectSpread4.default)({}, DEFAULT_API_STATE, filterKeys(state, refWhitelist, requestRefs), {
          inFlight: _inFlight
        });
        return newState;
      }

    case _apiActionCreators.API_RESP_SUCCESS: // fall though

    case _apiActionCreators.API_RESP_ERROR:
    case _cacheActionCreators.CACHE_SUCCESS:
      // fall through
      if (!action.payload) {
        throw new Error("".concat(action.type, " dispatched without required payload"));
      } // each of these actions provides an API response that should go into app
      // state - error responses will contain error info


      delete state.fail; // if there are any values, the API is not failing

      return (0, _objectSpread4.default)({}, state, responseToState(state, action.payload));

    case _apiActionCreators.API_RESP_FAIL:
      return (0, _objectSpread4.default)({}, state, {
        fail: action.payload
      });

    case _apiActionCreators.API_RESP_COMPLETE:
      {
        // always called - clean up inFlight
        var _refs = (action.payload || []).map(function (_ref6) {
          var ref = _ref6.ref;
          return ref;
        });

        var _inFlight2 = state.inFlight.filter(function (ref) {
          return !_refs.includes(ref);
        });

        return (0, _objectSpread4.default)({}, state, {
          inFlight: _inFlight2
        });
      }

    default:
      return state;
  }
}
//# sourceMappingURL=reducer.js.map