"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _mwpRouter = require("mwp-router");

var _reducer = require("./reducer");

var apiActions = _interopRequireWildcard(require("./sync/apiActionCreators"));

describe('getListState', function () {
  var state = {};
  var resp = {
    response: {
      value: ['foo']
    },
    query: {
      ref: 'bar',
      list: {
        dynamicRef: 'baz',
        merge: {
          idTest: function idTest() {
            return false;
          },
          sort: function sort(a, b) {
            if (a > b) {
              return 1;
            }

            if (a < b) {
              return -1;
            }

            return 0;
          }
        }
      }
    }
  };
  var respWithoutSort = {
    response: {
      value: ['foo']
    },
    query: {
      ref: 'bar',
      list: {
        dynamicRef: 'baz',
        merge: {
          idTest: function idTest() {
            return false;
          }
        }
      }
    }
  };
  var respWithReverse = {
    response: {
      value: ['foo']
    },
    query: {
      ref: 'bar',
      list: {
        dynamicRef: 'baz',
        merge: {
          idTest: function idTest() {
            return false;
          },
          isReverse: true
        }
      }
    }
  };
  it('ignores no-response responses', function () {
    expect((0, _reducer.getListState)(state, {
      response: null,
      query: resp.query
    })).toEqual({});
  });
  it('ignores no-list queries', function () {
    expect((0, _reducer.getListState)(state, {
      response: resp.response,
      query: {
        ref: 'bar'
      }
    })).toEqual({});
  });
  it('returns a new object with dynamicRef', function () {
    var ref = resp.query.list.dynamicRef;
    expect((0, _reducer.getListState)(state, resp)).toEqual((0, _defineProperty2.default)({}, ref, {
      value: resp.response.value,
      query: resp.query
    }));
  });
  it('merges new response with existing dynamicRef, sorted', function () {
    var value = ['qux'];
    var ref = resp.query.list.dynamicRef;
    expect((0, _reducer.getListState)((0, _defineProperty2.default)({}, ref, {
      value: value
    }), resp)).toEqual((0, _defineProperty2.default)({}, ref, {
      value: [].concat(value, (0, _toConsumableArray2.default)(resp.response.value)).sort(resp.query.list.merge.sort),
      query: resp.query
    }));
  });
  it('merges new response with existing dynamicRef, not sorted if sort callback undefined', function () {
    var value = ['qux'];
    var ref = respWithoutSort.query.list.dynamicRef;
    expect((0, _reducer.getListState)((0, _defineProperty2.default)({}, ref, {
      value: value
    }), respWithoutSort)).toEqual((0, _defineProperty2.default)({}, ref, {
      value: [].concat(value, (0, _toConsumableArray2.default)(respWithoutSort.response.value)),
      query: respWithoutSort.query
    }));
  });
  it('merges new response with existing dynamicRef adding new items at the top of the list', function () {
    var value = ['qux'];
    var ref = respWithReverse.query.list.dynamicRef;
    expect((0, _reducer.getListState)((0, _defineProperty2.default)({}, ref, {
      value: value
    }), respWithReverse)).toEqual((0, _defineProperty2.default)({}, ref, {
      value: [].concat((0, _toConsumableArray2.default)(respWithReverse.response.value), value),
      query: respWithReverse.query
    }));
  });
  it('merges new response with existing dynamicRef, sorted in case list is not top level value (is under `value` filed)', function () {
    var list = ['qux'];
    var value = {
      value: list
    };
    var response = {
      response: {
        value: {
          value: ['foo']
        }
      },
      query: resp.query
    };
    var ref = response.query.list.dynamicRef;
    expect((0, _reducer.getListState)((0, _defineProperty2.default)({}, ref, {
      value: list
    }), response)).toEqual((0, _defineProperty2.default)({}, ref, {
      value: [].concat((0, _toConsumableArray2.default)(value.value), (0, _toConsumableArray2.default)(response.response.value.value)).sort(response.query.list.merge.sort),
      query: response.query
    }));
  });
});
describe('api reducer', function () {
  it('returns default state for empty action', function () {
    expect((0, _reducer.api)((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE), {})).toEqual(_reducer.DEFAULT_API_STATE);
  });
  it('re-sets api state on logout API_REQ, with inFlight query', function () {
    var ref = 'foobar';
    var logoutRequest = apiActions.get([{
      ref: ref
    }], {
      logout: true
    });
    expect((0, _reducer.api)((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE), logoutRequest)).toEqual((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE, {
      inFlight: [ref]
    }));
  });
  it('clears refs corresponding to non-GET queries', function () {
    var bar = {
      query: {
        meta: {
          method: 'post'
        }
      }
    };
    var baz = {
      query: {
        meta: {
          method: 'get'
        }
      }
    };
    var populatedState = (0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE, {
      bar: bar,
      baz: baz
    });
    var action = {
      type: _mwpRouter.LOCATION_CHANGE
    };
    expect((0, _reducer.api)(populatedState, action)).toEqual((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE, {
      baz: baz
    }));
  });
  it('clears refs corresponding to new requests', function () {
    var ref = 'foobar';
    var populatedState = (0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE, {
      bar: 'something unrelated'
    });
    var populatedStateWithRef = (0, _objectSpread3.default)({}, populatedState, (0, _defineProperty2.default)({}, ref, 'not empty'));
    var action = apiActions.get([{
      ref: ref
    }]);
    expect((0, _reducer.api)(populatedStateWithRef, action)).toEqual((0, _objectSpread3.default)({}, populatedState, {
      inFlight: [ref]
    }));
  });
  it('adds success response to state tree', function () {
    var resp = {
      response: {
        ref: 'bing',
        value: 'baz'
      },
      query: {
        ref: 'bing'
      }
    };
    var API_RESP_SUCCESS = apiActions.success(resp);
    expect((0, _reducer.api)((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE, {
      foo: 'bar'
    }), API_RESP_SUCCESS)).toEqual({
      foo: 'bar',
      bing: {
        ref: 'bing',
        value: 'baz',
        query: resp.query
      },
      inFlight: []
    });
  });
  it('adds error response to state tree', function () {
    var resp = {
      response: {
        ref: 'bing',
        error: 'baz'
      },
      query: {
        ref: 'bing'
      }
    };
    var API_RESP_ERROR = apiActions.error(resp);
    expect((0, _reducer.api)((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE, {
      foo: 'bar'
    }), API_RESP_ERROR)).toEqual({
      foo: 'bar',
      bing: {
        ref: 'bing',
        error: 'baz',
        query: resp.query
      },
      inFlight: []
    });
  });
  it('populates an `fail` key on API_RESP_FAIL', function () {
    var API_RESP_FAIL = apiActions.fail(new Error('this is the worst'));
    var errorState = (0, _reducer.api)((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE), API_RESP_FAIL);
    expect(errorState).toEqual(expect.objectContaining({
      fail: API_RESP_FAIL.payload
    }));
  });
  it('adds query ref to inFlight array on API_REQ', function () {
    var ref = 'foobar';
    var API_REQ = apiActions.get([{
      ref: ref
    }]);
    var apiState = (0, _reducer.api)((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE), API_REQ);
    expect(apiState).toMatchObject({
      inFlight: [ref]
    });
  });
  it('removes query refs from inFlight array on API_RESP_COMPLETE', function () {
    var ref1 = 'foobar';
    var ref2 = 'barfoo';
    var query1 = {
      ref: ref1
    };
    var query2 = {
      ref: ref2
    };
    var inFlightState = [ref1, ref2, 'asdf'];
    var expectedInFlightState = ['asdf'];
    var completeAction = apiActions.complete([query1, query2]);
    var apiState = (0, _reducer.api)((0, _objectSpread3.default)({}, _reducer.DEFAULT_API_STATE, {
      inFlight: inFlightState
    }), completeAction);
    expect(apiState).toMatchObject({
      inFlight: expectedInFlightState
    });
  });
});
describe('filterKeys', function () {
  it('returns an object with all specified keys removed', function () {
    var orig = {
      foo: 'bar',
      baz: 'qux',
      so: 'what'
    };
    expect((0, _reducer.filterKeys)(orig, [], ['foo', 'so'])).toEqual({
      baz: 'qux'
    });
  });
  it('does not remove whitelisted keys when specified', function () {
    var orig = {
      foo: 'bar',
      baz: 'qux',
      so: 'what'
    };
    expect((0, _reducer.filterKeys)(orig, ['foo'], ['foo', 'so'])).toEqual({
      foo: 'bar',
      baz: 'qux'
    });
  });
});
//# sourceMappingURL=reducer.test.js.map