"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combineEpics = exports.createEpicMiddleware = void 0;

// type Epic = (action: Action, store: ReduxStore) => Promise<Array<Action>>
var createEpicMiddleware = function createEpicMiddleware(epic) {
  return function (store) {
    return function (next) {
      return function (action) {
        // need to execute epics in the next tick in order to ensure that that state
        // has been updated by reducers
        Promise.resolve().then(function () {
          return epic(action, store).then(function (actions) {
            actions.forEach(function (a) {
              return store.dispatch(a);
            });
          });
        });
        return next(action);
      };
    };
  };
};

exports.createEpicMiddleware = createEpicMiddleware;

var flattenArray = function flattenArray(arrays) {
  return [].concat.apply([], arrays);
};

var combineEpics = function combineEpics() {
  for (var _len = arguments.length, epics = new Array(_len), _key = 0; _key < _len; _key++) {
    epics[_key] = arguments[_key];
  }

  return function (action, store) {
    return Promise.all(epics.map(function (e) {
      return e(action, store);
    })).then(flattenArray);
  };
};

exports.combineEpics = combineEpics;
//# sourceMappingURL=index.js.map