"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.success = success;
exports.error = error;
exports.fail = fail;
exports.complete = complete;
exports.track = exports.del = exports.put = exports.patch = exports.post = exports.get = exports.API_RESP_FAIL = exports.API_RESP_ERROR = exports.API_RESP_COMPLETE = exports.API_RESP_SUCCESS = exports.API_REQ = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var API_REQ = 'API_REQ';
exports.API_REQ = API_REQ;
var API_RESP_SUCCESS = 'API_RESP_SUCCESS';
exports.API_RESP_SUCCESS = API_RESP_SUCCESS;
var API_RESP_COMPLETE = 'API_RESP_COMPLETE';
exports.API_RESP_COMPLETE = API_RESP_COMPLETE;
var API_RESP_ERROR = 'API_RESP_ERROR';
exports.API_RESP_ERROR = API_RESP_ERROR;
var API_RESP_FAIL = 'API_RESP_FAIL';
/**
 * Generic base action creator upon which all individual query action creators
 * are built. This action creator is designed to support parallel queries with
 * request metadata, which primarily appears in navigation-related actions.
 *
 * All request actions will get a Promise object injected into the `meta`
 * property, along with `resolve` and `reject` methods that can be used to
 * programmatically resolve the Promise. The expectation is that this Promise
 * will be used to indicate the state of the request, and will resolve when the
 * request is resolved or rejected.
 *
 * @param {Array} queries the queries to send to the server
 * @param {Object} meta metadata about the request, e.g. 'logout', 'clickTracking'
 * @return {Object} an API_REQ action
 */

exports.API_RESP_FAIL = API_RESP_FAIL;

function _requestAll(queries, meta) {
  if (process.env.NODE_ENV !== 'production') {
    // check queries have valid 'meta.method' value
    var method = queries[0].meta && queries[0].meta.method;
    queries.forEach(function (q) {
      if (!q.meta || !q.meta.method || q.meta.method !== method) {
        // meta.method must be set and must be the same for all queries
        console.error('_requestAll should not be called directly', 'use a method-specific API request action creator');
      }
    });
  }

  meta = meta || {};
  meta.request = new Promise(function (resolve, reject) {
    meta = meta || {};
    meta.resolve = resolve;
    meta.reject = reject;
  });
  return {
    type: API_REQ,
    payload: queries,
    meta: meta
  };
}

var setMethod = function setMethod(method) {
  return function (q) {
    if (process.env.NODE_ENV !== 'production') {
      // just some dev debug niceties that will be stripped out in prod
      var methodValue = (q.meta || {}).method;

      if (methodValue) {
        console.error(JSON.stringify(q));
        console.error('Query objects should not specify a `meta.method`');

        if (methodValue !== method) {
          throw new TypeError('query.meta.method does not match API action creator method');
        }
      }
    }

    return (0, _objectSpread2.default)({}, q, {
      meta: (0, _objectSpread2.default)({}, q.meta || {}, {
        method: method
      })
    });
  };
};

var _applyMethod = function _applyMethod(method) {
  return function (query, meta) {
    var queries = query instanceof Array ? query : [query]; // delegate to `_requestAll`

    return _requestAll(queries.map(setMethod(method)), meta);
  };
};

var get = _applyMethod('get');

exports.get = get;

var post = _applyMethod('post');

exports.post = post;

var patch = _applyMethod('patch');

exports.patch = patch;

var put = _applyMethod('put');

exports.put = put;

var del = _applyMethod('delete');

exports.del = del;

var track = function track(query, meta) {
  meta = meta || {};
  query.endpoint = 'track';
  meta.clickTracking = true;
  return post(query, meta);
};

exports.track = track;

function success(_ref) {
  var query = _ref.query,
      response = _ref.response;
  return {
    type: API_RESP_SUCCESS,
    payload: {
      query: query,
      response: response
    }
  };
}

function error(_ref2) {
  var query = _ref2.query,
      response = _ref2.response;
  return {
    type: API_RESP_ERROR,
    payload: {
      query: query,
      response: response
    }
  };
}

function fail(err) {
  return {
    type: API_RESP_FAIL,
    payload: err
  };
}

function complete(queries) {
  return {
    type: API_RESP_COMPLETE,
    payload: queries
  };
}
//# sourceMappingURL=apiActionCreators.js.map