"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var api = _interopRequireWildcard(require("./apiActionCreators"));

describe('method-specific creators', function () {
  it('populates the correct query.meta.method property', function () {
    var methodMap = {
      get: 'get',
      patch: 'patch',
      put: 'put',
      post: 'post',
      del: 'delete'
    };
    Object.keys(methodMap).forEach(function (key) {
      var query = {};
      var expectedQuery = {
        meta: {
          method: methodMap[key]
        }
      };
      var requestAction = api[key](query);
      expect(requestAction.type).toBe('API_REQ');
      expect(requestAction.payload).toHaveLength(1);
      expect(requestAction.payload[0]).toMatchObject(expectedQuery);
      expect(requestAction.meta.request).toEqual(expect.any(Promise));
      expect(requestAction.meta.resolve).toEqual(expect.any(Function));
      expect(requestAction.meta.reject).toEqual(expect.any(Function));
    });
  });
  it('track sets endpoint, meta.method, adds requestMeta.clickTracking.true', function () {
    var query = {};
    var trackAction = api.track(query);
    expect(trackAction.type).toBe('API_REQ');
    expect(trackAction.payload).toHaveLength(1);
    expect(trackAction.payload[0]).toMatchObject({
      endpoint: 'track',
      meta: {
        method: 'post'
      }
    });
    expect(trackAction.meta.clickTracking).toBe(true);
  });
});
//# sourceMappingURL=apiActionCreators.test.js.map