"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDeprecatedSuccessPayload = getDeprecatedSuccessPayload;
exports.default = exports.getFetchQueriesEpic = exports.apiRequestToApiReq = exports.getNavEpic = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _reduxPromiseEpic = require("../redux-promise-epic");

var _mwpRouter = require("mwp-router");

var _util = require("mwp-router/lib/util");

var api = _interopRequireWildcard(require("./apiActionCreators"));

var _syncActionCreators = require("./syncActionCreators");

var IGNORE_ACTION = Promise.resolve([]);
/**
 * @deprecated
 */

function getDeprecatedSuccessPayload(successes, errors) {
  var allQueryResponses = [].concat((0, _toConsumableArray2.default)(successes), (0, _toConsumableArray2.default)(errors));
  return allQueryResponses.reduce(function (payload, _ref) {
    var query = _ref.query,
        response = _ref.response;

    if (!response) {
      return payload;
    }

    var ref = response.ref,
        error = response.error,
        responseBody = (0, _objectWithoutProperties2.default)(response, ["ref", "error"]);

    if (error) {
      // old payload expects error as a property of `value`
      responseBody.value = {
        error: error
      };
    }

    payload.queries.push(query);
    payload.responses.push((0, _defineProperty2.default)({}, ref, responseBody));
    return payload;
  }, {
    queries: [],
    responses: []
  });
}
/**
 * Any operations that keep the browser application in sync with the
 * server should be implemented here.
 *
 * - Navigation-generated API request handling
 * - Arbitrary API request handling (syncActionCreators.apiSuccess)
 *
 * @module syncEpic
 */

/**
 * Navigation actions will provide the `location` as the payload, which this
 * epic will use to collect the current Reactive Queries associated with the
 * active routes.
 *
 * These queries will then be dispatched in the payload of `apiRequest`. Any
 * metadata about the navigation action can also be sent to the `apiRequest`
 * here.
 *
 * note that this function executes _downstream_ of reducers, so the
 * new `routing` data has already been populated in `state`
 *
 * @param {Object} routes The application's React Router routes
 * @returns {Function} an Epic function that emits an API_RESP_REQUEST action
 */


var getNavEpic = function getNavEpic(findMatches) {
  return function (action, store) {
    if (![_mwpRouter.LOCATION_CHANGE, _mwpRouter.SERVER_RENDER].includes(action.type)) {
      return IGNORE_ACTION;
    }

    var location = action.payload;
    var state = store.getState();
    var _state$routing$referr = state.routing.referrer,
        referrer = _state$routing$referr === void 0 ? {} : _state$routing$referr; // inject request metadata from context, including `store.getState()`

    var requestMetadata = {
      referrer: referrer.pathname || state.config.entryPath || '',
      logout: location.pathname.endsWith('logout'),
      // assume logout route ends with logout - not currently implemented in any app
      clickTracking: true,
      // clicks should be tracked with this request
      retainRefs: []
    };
    var cacheAction = requestMetadata.logout && {
      type: 'CACHE_CLEAR'
    };
    var previousQueries = referrer.pathname ? (0, _util.getMatchedQueries)(referrer, state)(findMatches(referrer)) : [];
    var newQueries = (0, _util.getMatchedQueries)(location, state)(findMatches(location));

    if (newQueries.filter(function (q) {
      return q;
    }).length === 0) {
      // no valid queries - jump straight to 'complete'
      return [api.complete([])];
    } // perform a fast comparison of previous route's serialized queries
    // with the new route's serialized queries. All state refs for
    // _shared_ queries should be retained


    var serializedNew = newQueries.map(JSON.stringify);
    var serializedPrev = previousQueries.map(JSON.stringify);
    var sharedRefs = serializedPrev.filter(function (qJSON) {
      return serializedNew.includes(qJSON);
    }).map(JSON.parse).map(function (q) {
      return q.ref;
    });
    requestMetadata.retainRefs = sharedRefs;
    return Promise.resolve([cacheAction, api.get(newQueries, requestMetadata)].filter(function (a) {
      return a;
    }));
  };
};
/**
 * Old apiRequest maps directly onto new api.get
 * @deprecated
 */


exports.getNavEpic = getNavEpic;

var apiRequestToApiReq = function apiRequestToApiReq(action) {
  return action.type === 'API_REQUEST' ? Promise.resolve([api.get(action.payload, action.meta)]) : IGNORE_ACTION;
};
/**
 * Listen for API_REQ and generate response actions from fetch results
 *
 * emits
 * - 1 or more API_RESP_SUCCESS
 * - 1 or more API_RESP_ERROR
 * - API_SUCCESS  // deprecated
 * - API_COMPLETE
 *
 * or
 *
 * - API_RESP_FAIL
 * - API_ERROR  // deprecated
 * - API_COMPLETE
 */


exports.apiRequestToApiReq = apiRequestToApiReq;

var getFetchQueriesEpic = function getFetchQueriesEpic(findMatches, fetchQueriesFn) {
  // keep track of location changes - will first be set by SERVER_RENDER
  var locationIndex = 0;

  var standardizeUrl = function standardizeUrl(location) {
    var matches = findMatches(location);
    return matches ? matches.pop().match.path.replace(/[^a-zA-Z0-9/]/gi, '') : '';
  }; // set up a closure that will compare the partially-applied location to the current value
  // of `locationIndex` - if `locationIndex` has changed since `currentLocation`
  // was passed in, return an empty array instead of the supplied actions array
  // This is a way of ignoring API return values that happen after a location change


  var ignoreIfLocationChange = function ignoreIfLocationChange(currentLocation) {
    return function (actions) {
      return currentLocation === locationIndex ? actions : [];
    };
  }; // return the epic


  return function (action, store) {
    if (action.type === _mwpRouter.LOCATION_CHANGE) {
      locationIndex = locationIndex + 1;
    }

    if (action.type !== api.API_REQ) {
      return IGNORE_ACTION;
    }

    var queries = action.payload,
        meta = action.meta; // set up the fetch call to the app server

    var _store$getState = store.getState(),
        config = _store$getState.config,
        self = _store$getState.api.self,
        _store$getState$routi = _store$getState.routing,
        location = _store$getState$routi.location,
        referrer = _store$getState$routi.referrer; // clean up path for use as endpoint URL


    var standardizedUrlPath = standardizeUrl(location);
    var standardizedReferrerPath = referrer && referrer.pathname ? standardizeUrl(referrer) : undefined; // construct the fetch call using match.path

    var fetchUrl = "".concat(config.apiUrl).concat(standardizedUrlPath);
    var fetchQueries = fetchQueriesFn(fetchUrl, (self || {}).value); // supply additional request info, e.g. for tracking

    var requestInfo = {
      standardized_url: standardizedUrlPath,
      standardized_referer: standardizedReferrerPath
    };
    return fetchQueries(queries, meta, requestInfo).then(function (_ref2) {
      var _ref2$successes = _ref2.successes,
          successes = _ref2$successes === void 0 ? [] : _ref2$successes,
          _ref2$errors = _ref2.errors,
          errors = _ref2$errors === void 0 ? [] : _ref2$errors;
      // meta contains a Promise that must be resolved
      meta.resolve([].concat((0, _toConsumableArray2.default)(successes), (0, _toConsumableArray2.default)(errors)));
      var deprecatedSuccessPayload = getDeprecatedSuccessPayload(successes, errors);
      var deprecatedActions = [(0, _syncActionCreators.apiSuccess)(deprecatedSuccessPayload)];

      if (meta && meta.onSuccess) {
        deprecatedActions.push(meta.onSuccess(deprecatedSuccessPayload));
      }

      return [].concat((0, _toConsumableArray2.default)(successes.map(api.success)), (0, _toConsumableArray2.default)(errors.map(api.error)), deprecatedActions);
    }).catch(function (err) {
      // meta contains a Promise that must be rejected
      meta.reject(err);
      var deprecatedActions = [(0, _syncActionCreators.apiError)(err)];

      if (meta && meta.onError) {
        deprecatedActions.push(meta.onError(err));
      }

      return [api.fail(err)].concat(deprecatedActions);
    }).then(ignoreIfLocationChange(locationIndex)).then(function (actions) {
      return [].concat((0, _toConsumableArray2.default)(actions), [api.complete(queries)]);
    });
  };
};

exports.getFetchQueriesEpic = getFetchQueriesEpic;

var _default = function _default(findMatches, fetchQueriesFn) {
  return (0, _reduxPromiseEpic.combineEpics)(getNavEpic(findMatches), getFetchQueriesEpic(findMatches, fetchQueriesFn), apiRequestToApiReq);
};

exports.default = _default;
//# sourceMappingURL=index.js.map