"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _mwpRouter = require("mwp-router");

var _mwpTestUtils = require("mwp-test-utils");

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

var _app = require("meetup-web-mocks/lib/app");

var _cacheActionCreators = require("../cache/cacheActionCreators");

var api = _interopRequireWildcard(require("./apiActionCreators"));

var syncActionCreators = _interopRequireWildcard(require("./syncActionCreators"));

var _ = _interopRequireWildcard(require("./"));

var _apiActionCreators2 = require("../../lib/sync/apiActionCreators");

global.fetch = _nodeFetch.default;
_app.MOCK_APP_STATE.config = {};
_app.MOCK_APP_STATE.routing = {};

var MAKE_MOCK_FIND_MATCHES = function MAKE_MOCK_FIND_MATCHES() {
  var queryFn = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : function () {
    return {
      params: {}
    };
  };
  return function () {
    return [{
      route: {
        path: '/',
        query: queryFn
      },
      match: {
        path: '/',
        params: {}
      }
    }];
  };
};
/**
 * @module SyncEpicTest
 */


describe('Sync epic', function () {
  it('does not emit actions for undefined action input', function () {
    return (0, _.default)(_app.MOCK_ROUTES)({
      type: 'asdf'
    }).then(function (actions) {
      return expect(actions).toHaveLength(0);
    });
  });
  describe('getNavEpic', function () {
    it('emits API_REQ for nav-related actions with matched query', function () {
      var locationChange = {
        type: _mwpRouter.LOCATION_CHANGE,
        payload: _app.MOCK_RENDERPROPS.location
      };
      var serverRender = {
        type: _mwpRouter.SERVER_RENDER,
        payload: _app.MOCK_RENDERPROPS.location
      };
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      var navEpic = (0, _.getNavEpic)(MAKE_MOCK_FIND_MATCHES());
      return Promise.all([navEpic(locationChange, fakeStore), navEpic(serverRender, fakeStore)]).then(function (actionArrays) {
        actionArrays.forEach(function (actions) {
          var types = actions.map(function (a) {
            return a.type;
          });
          expect(types).toContain(api.API_REQ);
        });
      });
    });
    it('emits API_REQ, CACHE_CLEAR for nav-related actions with logout request', function () {
      var logoutLocation = (0, _objectSpread2.default)({}, _app.MOCK_RENDERPROPS.location, {
        pathname: '/logout'
      });
      var locationChange = {
        type: _mwpRouter.LOCATION_CHANGE,
        payload: logoutLocation
      };
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      return (0, _.getNavEpic)(MAKE_MOCK_FIND_MATCHES())(locationChange, fakeStore).then(function (actions) {
        var types = actions.map(function (a) {
          return a.type;
        });
        expect(types).toContain(api.API_REQ);
        expect(types.includes(_cacheActionCreators.CACHE_CLEAR)).toBe(true);
      });
    });
    it('emits API_COMPLETE for nav-related actions without matched query', function () {
      var pathname = '/noQuery';
      var noMatchLocation = (0, _objectSpread2.default)({}, _app.MOCK_RENDERPROPS.location, {
        pathname: pathname
      });
      var locationChange = {
        type: _mwpRouter.LOCATION_CHANGE,
        payload: noMatchLocation
      };
      var serverRender = {
        type: _mwpRouter.SERVER_RENDER,
        payload: noMatchLocation
      };
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      var navEpic = (0, _.getNavEpic)(MAKE_MOCK_FIND_MATCHES(null));
      return Promise.all([navEpic(locationChange, fakeStore), navEpic(serverRender, fakeStore)]).then(function (actionArrays) {
        actionArrays.forEach(function (actions) {
          expect(actions.map(function (_ref) {
            var type = _ref.type;
            return type;
          })).toEqual([_apiActionCreators2.API_RESP_COMPLETE]);
        });
      });
    });
    it('emits API_COMPLETE for nav-related actions with query functions that return null', function () {
      var pathname = '/nullQuery';
      var noMatchLocation = (0, _objectSpread2.default)({}, _app.MOCK_RENDERPROPS.location, {
        pathname: pathname
      });
      var locationChange = {
        type: _mwpRouter.LOCATION_CHANGE,
        payload: noMatchLocation
      };
      var serverRender = {
        type: _mwpRouter.SERVER_RENDER,
        payload: noMatchLocation
      };
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      var navEpic = (0, _.getNavEpic)(MAKE_MOCK_FIND_MATCHES(function () {
        return null;
      }));
      return Promise.all([navEpic(locationChange, fakeStore), navEpic(serverRender, fakeStore)]).then(function (actionArrays) {
        actionArrays.forEach(function (actions) {
          expect(actions.map(function (_ref2) {
            var type = _ref2.type;
            return type;
          })).toEqual([_apiActionCreators2.API_RESP_COMPLETE]);
        });
      });
    });
  });
  describe('getFetchQueriesEpic', function () {
    it('emits API_RESP_SUCCESS and API_RESP_COMPLETE on successful API_REQ', function () {
      var mockFetchQueries = function mockFetchQueries() {
        return function () {
          return Promise.resolve({
            successes: [{}]
          });
        };
      };

      var queries = [(0, _app.mockQuery)({})];
      var apiRequest = api.get(queries);
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      var fqEpic = (0, _.getFetchQueriesEpic)(MAKE_MOCK_FIND_MATCHES(), mockFetchQueries); // kick off the fetch

      var doFetch = fqEpic(apiRequest, fakeStore);
      return doFetch.then(function (actions) {
        expect(actions.map(function (_ref3) {
          var type = _ref3.type;
          return type;
        })).toEqual([api.API_RESP_SUCCESS, 'API_SUCCESS', api.API_RESP_COMPLETE]);
      });
    });
    it('does not emit API_RESP_SUCCESS when API_REQ is interrupted by LOCATION_CHANGE', function () {
      // mock that takes 10ms to return (allows time for LOCATION_CHANGE to interrupt)
      var mockFetchQueries = function mockFetchQueries() {
        return function () {
          return new Promise(function (resolve, reject) {
            return setTimeout(function () {
              return resolve({
                successes: [{}]
              });
            }, 10);
          });
        };
      };

      var queries = [(0, _app.mockQuery)({})];
      var apiRequest = api.get(queries);
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      var fqEpic = (0, _.getFetchQueriesEpic)(MAKE_MOCK_FIND_MATCHES(), mockFetchQueries); // kick off the fetch

      var doFetch = fqEpic(apiRequest, fakeStore); // immediately trigger a location change

      fqEpic({
        type: _mwpRouter.LOCATION_CHANGE
      }, fakeStore);
      return doFetch.then(function (actions) {
        expect(actions.map(function (_ref4) {
          var type = _ref4.type;
          return type;
        })).not.toContain(api.API_RESP_SUCCESS);
      });
    });
    it('emits API_RESP_COMPLETE when API_REQ is interrupted by LOCATION_CHANGE', function () {
      // mock that takes 10ms to return (allows time for LOCATION_CHANGE to interrupt)
      var mockFetchQueries = function mockFetchQueries() {
        return function () {
          return new Promise(function (resolve, reject) {
            return setTimeout(function () {
              return resolve({
                successes: [{}]
              });
            }, 10);
          });
        };
      };

      var queries = [(0, _app.mockQuery)({})];
      var apiRequest = api.get(queries);
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      var fqEpic = (0, _.getFetchQueriesEpic)(MAKE_MOCK_FIND_MATCHES(), mockFetchQueries); // kick off the fetch

      var doFetch = fqEpic(apiRequest, fakeStore); // immediately trigger a location change

      fqEpic({
        type: _mwpRouter.LOCATION_CHANGE
      }, fakeStore);
      return doFetch.then(function (actions) {
        expect(actions.map(function (_ref5) {
          var type = _ref5.type;
          return type;
        })).toContain(api.API_RESP_COMPLETE);
      });
    });
    it('calls action.meta.resolve successful API_REQ', function () {
      var expectedSuccesses = [{}];

      var mockFetchQueries = function mockFetchQueries() {
        return function () {
          return Promise.resolve({
            successes: expectedSuccesses
          });
        };
      };

      var queries = [(0, _app.mockQuery)({})];
      var apiRequest = api.get(queries);
      apiRequest.meta.resolve = jest.fn();
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      return (0, _.getFetchQueriesEpic)(MAKE_MOCK_FIND_MATCHES(), mockFetchQueries)(apiRequest, fakeStore).then(function (actions) {
        expect(apiRequest.meta.resolve).toHaveBeenCalledWith(expectedSuccesses);
      });
    });
    it('emits API_RESP_FAIL on failed API_REQ', function () {
      var mockFetchQueries = function mockFetchQueries() {
        return function () {
          return Promise.reject(new Error('mock error'));
        };
      };

      var queries = [(0, _app.mockQuery)({})];
      var apiRequest = api.get(queries);
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      return (0, _.getFetchQueriesEpic)(MAKE_MOCK_FIND_MATCHES(), mockFetchQueries)(apiRequest, fakeStore).then(function (actions) {
        expect(actions.map(function (a) {
          return a.type;
        })).toEqual([api.API_RESP_FAIL, 'API_ERROR', api.API_RESP_COMPLETE]);
      });
    });
    it('calls action.meta.reject on failed API_REQ', function () {
      var expectedError = new Error();

      var mockFetchQueries = function mockFetchQueries() {
        return function () {
          return Promise.reject(expectedError);
        };
      };

      var queries = [(0, _app.mockQuery)({})];
      var apiRequest = api.get(queries);
      apiRequest.meta.reject = jest.fn();
      var fakeStore = (0, _mwpTestUtils.createFakeStore)(_app.MOCK_APP_STATE);
      return (0, _.getFetchQueriesEpic)(MAKE_MOCK_FIND_MATCHES(), mockFetchQueries)(apiRequest, fakeStore).then(function (actions) {
        return expect(apiRequest.meta.reject).toHaveBeenCalledWith(expectedError);
      });
    });
  });
});
describe('DEPRECATED apiRequestToApiReq', function () {
  it('emits API_REQ for API_REQUEST', function () {
    var queries = [(0, _app.mockQuery)({})];
    var apiRequest = syncActionCreators.apiRequest(queries);
    return (0, _.apiRequestToApiReq)(apiRequest).then(function (actions) {
      expect(actions).toHaveLength(1);
      expect(actions[0].type).toBe(api.API_REQ);
    });
  });
});
//# sourceMappingURL=index.test.js.map